varying mediump vec4 var_position;
varying mediump vec4 var_position_world;
varying highp vec3 var_normal;
varying mediump vec2 var_texcoord0;
varying mediump vec4 var_texcoord0_shadow;
varying mediump vec4 var_light;

uniform lowp sampler2D tex0;
uniform lowp sampler2D tex_depth;

uniform mediump vec4 mtx_light_mvp0;
uniform mediump vec4 mtx_light_mvp1;
uniform mediump vec4 mtx_light_mvp2;
uniform mediump vec4 mtx_light_mvp3;

uniform lowp vec4 fog_colour;
uniform mediump vec4 light;

float rgba_to_float(vec4 rgba)
{
    return dot(rgba, vec4(1.0, 1.0/255.0, 1.0/65025.0, 1.0/16581375.0));
}

float get_fraction(float value, float min, float max)
{
    return clamp((value-max)/(min-max), 0.0, 1.0);
}

void main()
{

    // Shadow 
    vec4 color = texture2D(tex0, var_texcoord0.xy);
    vec4 depth_proj = var_texcoord0_shadow / var_texcoord0_shadow.w;
    float shadow_alpha = 0.0;
    vec4 depth_data = vec4(depth_proj.xyzw);
    float depth = rgba_to_float(texture2D(tex_depth, depth_data.st));
    const float depth_bias = 0.005;
    float distance = abs(var_position.y);

    if (depth > depth_data.z - depth_bias)
    {
        float min = 0.0;
        float max = 2000.0;
        float fraction = get_fraction(distance, min, max);

        // Avoids shadows on walls
        float depth_min = 0.0;
        float depth_max = -1.0;
        float depth_fraction = get_fraction(var_position_world.z, depth_min, depth_max);

        fraction *= depth_fraction;
        
        shadow_alpha = fraction * 0.5; // Multiplied by alpha
    }
    if (shadow_alpha > 0.0)
    {
        vec4 shadow_colour = color * color;
        float darkness_fraction = 0.6; // Shadow darkness fraction (affecting light surfaces, 0 = black 1 = no shadow on white surfaces)
        shadow_colour.a = shadow_alpha ;
        shadow_colour.rgb *= shadow_colour.a * darkness_fraction;
        color.rgb *= color.a;
        color.rgb = shadow_colour.rgb + (color.rgb * (1.0 - shadow_colour.a));   
        color.a = shadow_colour.a + (color.a * (1.0 - shadow_colour.a));

    }

    gl_FragColor = color;
}

    